@@ -1,6 +1,6 @@
 ;;; Guile VM frame functions
 
-;;; Copyright (C) 2001, 2005, 2009-2016 Free Software Foundation, Inc.
+;;; Copyright (C) 2001, 2005, 2009-2016, 2018 Free Software Foundation, Inc.
 ;;;
 ;;; This library is free software; you can redistribute it and/or
 ;;; modify it under the terms of the GNU Lesser General Public
@@ -37,8 +37,8 @@
             frame-lookup-binding
             binding-ref binding-set!
 
-            frame-instruction-pointer-or-primitive-procedure-name
             frame-call-representation
+            frame-return-values
             frame-environment
             frame-object-binding frame-object-name))
 
@@ -103,6 +103,9 @@
     (define (find-idx n diff)
       (let lp ((n n) (diff diff))
         (cond
+         ((= n (vector-length parsed))
+          ;; Possible for jumps to alternate arities.
+          #f)
          ((negative? diff)
           (lp (1- n) (+ diff (vector-ref parsed (1- n)))))
          ((positive? diff)
@@ -202,9 +205,9 @@
         (for-each (lambda (slot)
                     (when (< slot (vector-length defs-by-slot))
                       (kill-slot! n slot)))
-                  (instruction-slot-clobbers code pos
-                                             (vector-ref in-sizes n)
-                                             (vector-ref out-sizes n)))
+                  (let ((in (vector-ref in-sizes n))
+                        (out (vector-ref out-sizes n)))
+                    (instruction-slot-clobbers code pos in out)))
         (lp (1+ n) (+ pos (vector-ref parsed n)))))
     killv))
 
@@ -322,24 +325,9 @@
 (define* (frame-procedure-name frame #:key
                                (info (find-program-debug-info
                                       (frame-instruction-pointer frame))))
-  (cond
-   (info => program-debug-info-name)
-   ;; We can only try to get the name from the closure if we know that
-   ;; slot 0 corresponds to the frame's procedure.  This isn't possible
-   ;; to know in general.  If the frame has already begun executing and
-   ;; the closure binding is dead, it could have been replaced with any
-   ;; other random value, or an unboxed value.  Even if we're catching
-   ;; the frame at its application, before it has started running, if
-   ;; the callee is well-known and has only one free variable, closure
-   ;; optimization could have chosen to represent its closure as that
-   ;; free variable, and that free variable might be some other program,
-   ;; or even an unboxed value.  It would be an error to try to get the
-   ;; procedure name of some procedure that doesn't correspond to the
-   ;; one being applied.  (Free variables are currently always boxed but
-   ;; that could change in the future.)
-   ((primitive-code? (frame-instruction-pointer frame))
-    (procedure-name (frame-local-ref frame 0 'scm)))
-   (else #f)))
+  (if info
+      (program-debug-info-name info)
+      (primitive-code-name (frame-instruction-pointer frame))))
 
 ;; This function is always called to get some sort of representation of the
 ;; frame to present to the user, so let's do the logical thing and dispatch to
@@ -347,23 +335,12 @@
 (define (frame-arguments frame)
   (cdr (frame-call-representation frame)))
 
-;; Usually the IP is sufficient to identify the procedure being called.
-;; However all primitive applications of the same arity share the same
-;; code.  Perhaps we should change that in the future, but for now we
-;; export this function to avoid having to export frame-local-ref.
-;;
-(define (frame-instruction-pointer-or-primitive-procedure-name frame)
-  (let ((ip (frame-instruction-pointer frame)))
-    (if (primitive-code? ip)
-        (procedure-name (frame-local-ref frame 0 'scm))
-        ip)))
-
 
 ;;;
 ;;; Pretty printing
 ;;;
 
-;; Basically there are two cases to deal with here:
+;; Basically there are three cases to deal with here:
 ;;
 ;;   1. We've already parsed the arguments, and bound them to local
 ;;      variables. In a standard (lambda (a b c) ...) call, this doesn't
@@ -377,6 +354,10 @@
 ;;      number of arguments, or perhaps we're doing a typed dispatch and
 ;;      the types don't match. In that case the arguments are all on the
 ;;      stack, and nothing else is on the stack.
+;;
+;;   3. Alternately it's possible that we're between a primitive call
+;;      and its associated return.  In that case, we won't be able to
+;;      say anything at all.
 
 (define* (frame-call-representation frame #:key top-frame?)
   (let* ((ip (frame-instruction-pointer frame))
